package de.ozgcloud.nachrichten.postfach.osiv2;

import java.util.stream.Stream;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.PostfachRemoteService;
import de.ozgcloud.nachrichten.postfach.osiv2.exception.Osi2ExceptionHandler;
import de.ozgcloud.nachrichten.postfach.osiv2.exception.Osi2PostfachException;
import de.ozgcloud.nachrichten.postfach.osiv2.transfer.Osi2PostfachService;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@ServiceIfOsi2Enabled
@Log4j2
@RequiredArgsConstructor
public class OsiPostfachRemoteService implements PostfachRemoteService {
	private final Osi2PostfachService osi2PostfachService;
	private final Osi2ExceptionHandler exceptionHandler;

	public static final String POSTFACH_TYPE_OSI = "OSI";

	@Override
	public void sendMessage(PostfachNachricht nachricht) {
		try {
			osi2PostfachService.sendMessage(nachricht);
		} catch (RuntimeException e) {
			throw new Osi2PostfachException("sendMessage failed!", exceptionHandler.deriveMessageCode(e), e);
		}
	}

	@Override
	public Stream<PostfachNachricht> getAllMessages() {
		try {
			return osi2PostfachService.receiveMessages();
		} catch (RuntimeException e) {
			throw new Osi2PostfachException("getAllMessages failed!", exceptionHandler.deriveMessageCode(e), e);
		}
	}

	@Override
	public void deleteMessage(String messageId) {
		try {
			osi2PostfachService.deleteMessage(messageId);
		} catch (RuntimeException e) {
			throw new Osi2PostfachException("deleteMessage failed!", exceptionHandler.deriveMessageCode(e), e);
		}
	}

	@Override
	public String getPostfachType() {
		return POSTFACH_TYPE_OSI;
	}

	@Override
	public boolean isReplyAllowed() {
		return true;
	}
}
