package de.ozgcloud.nachrichten.postfach.osiv2.attachment;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
class ConcatInputStream extends InputStream {
	private final Iterator<? extends InputStream> streams;

	private InputStream currentStream = null;

	@Override
	public int read() throws IOException {
		var buffer = new byte[1];
		var result = read(buffer, 0, 1);
		return result < 0 ? result : buffer[0];
	}

	private boolean useNextStreamFails() throws IOException {
		close();
		if (streams.hasNext()) {
			currentStream = streams.next();
			return false;
		}
		return true;
	}

	@Override
	public int read(byte[] buffer, int offset, int length) throws IOException {
		if (currentStream == null && useNextStreamFails()) {
			return -1;
		}
		var remainingLength = length;
		while (remainingLength > 0) {
			var readLength = currentStream.read(buffer, offset, remainingLength);
			if (readLength > 0) {
				remainingLength -= readLength;
				offset += readLength;
			} else if (useNextStreamFails()) {
				break;
			}
		}
		var totalReadLength = length - remainingLength;
		return totalReadLength >= 0 ? totalReadLength : -1;
	}

	@Override
	public void close() throws IOException {
		if (currentStream != null) {
			currentStream.close();
			currentStream = null;
		}
	}
}
