package de.ozgcloud.nachrichten.postfach.osiv2.attachment;

import java.io.IOException;
import java.io.InputStream;

import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class LimitedInputStream extends InputStream {
	private final InputStream parentStream;
	private final long limit;
	private long readOffset = 0;

	@Override
	public int read() throws IOException {
		var buffer = new byte[1];
		var result = read(buffer, 0, 1);
		return result < 0 ? result : buffer[0];
	}

	@Override
	public int read(byte[] buffer, int offset, int length) throws IOException {
		var remainingLength = limit - readOffset;
		if (remainingLength > 0) {
			var readLength = parentStream.read(buffer, offset, (int) Math.min(remainingLength, length));
			if (readLength > 0) {
				readOffset += readLength;
				return readLength;
			}
		}
		return -1;
	}

	@Override
	public void close() {
		// Keep parent stream open
	}
}
