package de.ozgcloud.nachrichten.postfach.osiv2.attachment;

import static de.ozgcloud.nachrichten.postfach.osiv2.config.Osi2PostfachProperties.*;

import java.io.InputStream;
import java.util.Iterator;
import java.util.List;

import org.springframework.beans.factory.annotation.Qualifier;

import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.apilib.file.OzgCloudFile;
import de.ozgcloud.apilib.file.OzgCloudFileService;
import de.ozgcloud.nachrichten.postfach.osiv2.ServiceIfOsi2Enabled;
import de.ozgcloud.nachrichten.postfach.osiv2.model.AttachmentFile;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcBinaryFilesRequest;
import de.ozgcloud.vorgang.grpc.binaryFile.GrpcGetBinaryFileDataRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Log4j2
@ServiceIfOsi2Enabled
@RequiredArgsConstructor
public class Osi2AttachmentFileService {

	private BinaryFileServiceGrpc.BinaryFileServiceBlockingStub binaryFileServiceStub;
	private final Osi2AttachmentFileMapper attachmentFileMapper;
	@Qualifier(OZG_CLOUD_FILE_SERVICE_NAME)
	private final OzgCloudFileService ozgCloudFileService;
	private final OzgCloudCallContextProvider ozgCloudCallContextProvider;

	@GrpcClient(GRPC_FILE_MANAGER_NAME)
	public void setBinaryFileServiceStub(BinaryFileServiceGrpc.BinaryFileServiceBlockingStub binaryFilServiceStubWithoutInterceptor) {
		this.binaryFileServiceStub = binaryFilServiceStubWithoutInterceptor
				.withInterceptors(new OzgCloudCallContextAttachingInterceptor(ozgCloudCallContextProvider));
	}

	public List<OzgCloudFile> getFileMetadataOfIds(List<String> fileIds) {
		return binaryFileServiceStub.findBinaryFilesMetaData(createRequestWithFileIds(fileIds))
				.getFileList()
				.stream()
				.map(attachmentFileMapper::mapToUploadFileMetadata)
				.toList();
	}

	GrpcBinaryFilesRequest createRequestWithFileIds(List<String> fileIds) {
		return GrpcBinaryFilesRequest.newBuilder().addAllFileId(fileIds).build();
	}

	public String uploadFileAndReturnId(AttachmentFile attachmentFile, InputStream fileInputStream) {
		var attachmentFileId = ozgCloudFileService.uploadFile(
				attachmentFileMapper.toOzgCloudUploadFile(attachmentFile),
				fileInputStream
		);
		return attachmentFileId.toString();
	}

	public InputStream downloadFileContent(String fileId) {
		var response = binaryFileServiceStub.getBinaryFileContent(createRequestWithFileId(fileId));
		return new ConcatInputStream(new Iterator<>() {
			@Override
			public boolean hasNext() {
				return response.hasNext();
			}

			@Override
			public InputStream next() {
				return response.next().getFileContent().newInput();
			}
		});
	}

	GrpcGetBinaryFileDataRequest createRequestWithFileId(String fileId) {
		return GrpcGetBinaryFileDataRequest.newBuilder().setFileId(fileId).build();
	}

}
