package de.ozgcloud.nachrichten.postfach.osiv2.attachment;

import java.io.IOException;
import java.util.List;

import org.springframework.core.io.Resource;

import de.ozgcloud.nachrichten.postfach.osiv2.model.AttachmentFile;
import de.ozgcloud.nachrichten.postfach.osiv2.ServiceIfOsi2Enabled;
import de.ozgcloud.nachrichten.postfach.osiv2.exception.Osi2RuntimeException;
import de.ozgcloud.nachrichten.postfach.osiv2.model.AttachmentInfo;
import de.ozgcloud.nachrichten.postfach.osiv2.model.Osi2Message;
import de.ozgcloud.nachrichten.postfach.osiv2.transfer.PostfachApiFacadeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@ServiceIfOsi2Enabled
@RequiredArgsConstructor
public class Osi2PersistAttachmentService {
	private final PostfachApiFacadeService postfachApiFacadeService;
	private final Osi2AttachmentFileService attachmentFileService;

	public List<String> persistAttachments(Osi2Message osi2Message) {
		return osi2Message.attachments().stream()
				.map(info -> persistAttachment(osi2Message, info))
				.toList();
	}

	String persistAttachment(Osi2Message osi2Message, AttachmentInfo attachment) {
		return persistAttachmentFile(
				buildAttachmentFile(osi2Message.vorgangId(), attachment),
				postfachApiFacadeService.downloadAttachment(osi2Message.messageId(), attachment.guid())
		);
	}

	AttachmentFile buildAttachmentFile(String vorgangId, AttachmentInfo attachment) {
		return AttachmentFile.builder()
				.name(attachment.name())
				.contentType(attachment.contentType())
				.vorgangId(vorgangId)
				.build();
	}

	String persistAttachmentFile(AttachmentFile attachmentFile, Resource content) {
		try (var inputStream = content.getInputStream()) {
			return attachmentFileService.uploadFileAndReturnId(attachmentFile, inputStream);
		} catch (IOException e) {
			throw new Osi2RuntimeException("Error while persisting attachment", e);
		}
	}
}
