package de.ozgcloud.nachrichten.postfach.osiv2.config;

import java.util.List;

import jakarta.annotation.Nullable;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;

import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
@Configuration
@ConfigurationProperties(prefix = Osi2PostfachProperties.PREFIX)
public class Osi2PostfachProperties {
	// From de.ozgcloud.nachrichten.NachrichtenManagerConfiguration
	public static final String GRPC_FILE_MANAGER_NAME = "file-manager";
	// From de.ozgcloud.nachrichten.NachrichtenManagerConfiguration
	public static final String OZG_CLOUD_FILE_SERVICE_NAME = "nachrichten_OzgCloudFileService";

	public static final String PREFIX = "ozgcloud.osiv2";

	private boolean enabled;

	@Getter
	@Setter
	@Configuration
	@ConfigurationProperties(prefix = AuthConfiguration.PREFIX)
	public static class AuthConfiguration {
		public static final String PREFIX = Osi2PostfachProperties.PREFIX + ".auth";

		@NotBlank
		private String clientId;
		@NotBlank
		private String clientSecret;
		@NotNull
		@Valid
		private List<@NotBlank String> scope;
		@NotBlank
		private String tokenUri;
		@NotBlank
		private String resource;
	}

	@Getter
	@Setter
	@Configuration
	@ConfigurationProperties(prefix = ApiConfiguration.PREFIX)
	public static class ApiConfiguration {
		public static final String PREFIX = Osi2PostfachProperties.PREFIX + ".api";

		@NotBlank
		private String url;
		@NotBlank
		private String tenant;
		@NotBlank
		private String nameIdentifier;
	}

	/**
	 * HTTP proxy configuration. To use basic HTTP authentication configure username and password. (See <a
	 * href="https://github.com/reactor/reactor-netty/blob/3b74fcf2f5b2bdd129636e9c55608f0187ef7bd4/reactor-netty-core/src/main/java/reactor/netty/transport/ProxyProvider.java#L142">ProxyProvider</a>)
	 */
	@Getter
	@Setter
	@Configuration
	@ConfigurationProperties(prefix = ProxyConfiguration.PREFIX)
	public static class ProxyConfiguration {
		public static final String PREFIX = Osi2PostfachProperties.PREFIX + ".proxy";

		private boolean enabled;

		@NotBlank
		private String host;
		@NotNull
		private Integer port;

		@Nullable
		private String username = null;
		@Nullable
		private String password = null;
	}
}
