package de.ozgcloud.nachrichten.postfach.osiv2.exception;

import jakarta.annotation.Nullable;

import org.springframework.http.HttpStatusCode;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestClientResponseException;

import de.ozgcloud.nachrichten.postfach.PostfachMessageCode;
import de.ozgcloud.nachrichten.postfach.osiv2.ServiceIfOsi2Enabled;

@ServiceIfOsi2Enabled
public class Osi2ExceptionHandler {

	public PostfachMessageCode deriveMessageCode(@Nullable RuntimeException exception) {
		if (exception instanceof RestClientResponseException restClientResponseException) {
			return deriveMessageCodeFromRestClientResponseException(restClientResponseException);
		}
		if (exception instanceof ResourceAccessException) {
			return PostfachMessageCode.SERVER_CONNECTION_FAILED_MESSAGE_CODE;
		}
		if (exception instanceof Osi2MailboxNotFoundException) {
			return PostfachMessageCode.SEND_FAILED_UNKNOWN_POSTFACH_ID_MESSAGE_CODE;
		}
		return PostfachMessageCode.PROCESS_FAILED_MESSAGE_CODE;
	}

	PostfachMessageCode deriveMessageCodeFromRestClientResponseException(RestClientResponseException restClientResponseException) {
		if (hasStatusNotFound(restClientResponseException)) {
			return PostfachMessageCode.SEND_FAILED_UNKNOWN_POSTFACH_ID_MESSAGE_CODE;
		}
		return PostfachMessageCode.PROCESS_FAILED_MESSAGE_CODE;
	}

	private boolean hasStatusNotFound(RestClientResponseException restClientResponseException) {
		return restClientResponseException.getStatusCode().isSameCodeAs(HttpStatusCode.valueOf(404));
	}

}
