package de.ozgcloud.nachrichten.postfach.osiv2.model;

import static de.ozgcloud.nachrichten.postfach.osiv2.model.Osi2Attachment.*;

import java.io.InputStream;

import org.springframework.core.io.AbstractResource;

import de.ozgcloud.nachrichten.postfach.osiv2.attachment.LimitedInputStream;
import lombok.Builder;

@Builder
public record FileChunkInfo(
		Osi2Attachment upload,
		long chunkIndex
) {
	public AbstractResource createUploadResource(InputStream fileInputStream) {
		return new AbstractResource() {

			@Override
			public String getFilename() {
				return upload.file().getName();
			}

			@Override
			public String getDescription() {
				return "File chunk " + chunkIndex + " of " + upload.getLoggableString();
			}

			@Override
			public InputStream getInputStream() {
				return new LimitedInputStream(fileInputStream, contentLength());
			}

			@Override
			public long contentLength() {
				return chunkIndex == upload.numberOfChunks() ? 0 : getChunkContentLength();
			}

			private long getChunkContentLength() {
				return chunkIndex == upload.numberOfChunks() - 1
						? upload.file().getSize() % CHUNK_SIZE
						: CHUNK_SIZE;
			}
		};
	}
}
