package de.ozgcloud.nachrichten.postfach.osiv2.model;

import java.util.UUID;

import de.ozgcloud.apilib.file.OzgCloudFile;
import lombok.Builder;

@Builder
public record Osi2Attachment(
		String guid,
		OzgCloudFile file
) {
	public static final long CHUNK_SIZE = (2L << 14);

	public static Osi2Attachment from(OzgCloudFile file) {
		return Osi2Attachment.builder()
				.guid(UUID.randomUUID().toString())
				.file(file)
				.build();
	}

	public long numberOfChunks() {
		return (file.getSize() + CHUNK_SIZE - 1) / CHUNK_SIZE;
	}

	public String getLoggableString() {
		return String.format("Attachment(messageGuid=%s, ozgFileId=%s, size=%d)", guid, file.getId(), file.getSize());
	}
}
