package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import java.util.stream.Stream;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.ServiceIfOsi2Enabled;
import de.ozgcloud.nachrichten.postfach.osiv2.attachment.Osi2PersistAttachmentService;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@ServiceIfOsi2Enabled
@RequiredArgsConstructor
public class Osi2PostfachService {
	private final PostfachApiFacadeService postfachApiFacadeService;
	private final Osi2QuarantineService quarantineService;
	private final Osi2PersistAttachmentService persistAttachmentService;
	private final Osi2MessageMapper messageMapper;

	public void sendMessage(PostfachNachricht nachricht) {
		postfachApiFacadeService.sendMessage(
				messageMapper.mapOsi2Message(
						nachricht,
						postfachApiFacadeService.lookupMailboxId(messageMapper.getNameIdentifier(nachricht)),
						quarantineService.uploadAttachments(nachricht.getAttachments())
				)
		);
	}

	public Stream<PostfachNachricht> receiveMessages() {
		return postfachApiFacadeService.fetchPendingMessageIds()
				.stream()
				.map(this::fetchPostfachNachricht);
	}

	PostfachNachricht fetchPostfachNachricht(String messageGuid) {
		var message = postfachApiFacadeService.fetchMessageById(messageGuid);
		return messageMapper.mapPostfachNachricht(
				message,
				persistAttachmentService.persistAttachments(message)
		);
	}

	public void deleteMessage(final String messageId) {
		postfachApiFacadeService.deleteMessage(messageId);
	}

}
