package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import java.util.Collections;
import java.util.List;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.DomainChunkMetaData;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeFiles;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.OutSendMessageRequestV2;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1References;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyBehavior;
import de.ozgcloud.nachrichten.postfach.osiv2.model.AttachmentInfo;
import de.ozgcloud.nachrichten.postfach.osiv2.model.FileChunkInfo;
import de.ozgcloud.nachrichten.postfach.osiv2.model.Osi2Message;

@Mapper(unmappedTargetPolicy = ReportingPolicy.ERROR)
public interface Osi2RequestMapper {

	int MAX_NUMBER_RECEIVED_MESSAGES = 100;

	@Mapping(target = "sequencenumber", source = "vorgangId")
	@Mapping(target = "body", source = "mailBody")
	@Mapping(target = "displayName", ignore = true)
	@Mapping(target = "originSender", ignore = true)
	@Mapping(target = "replyAction", source = "replyOption")
	@Mapping(target = "eidasLevel", constant = "LOW")
	@Mapping(target = "isObligatory", expression = "java( false )")
	@Mapping(target = "isHtml", expression = "java( false )")
	@Mapping(target = "files", source = "attachments", qualifiedByName = "mapMessageExchangeFile")
	@Mapping(target = "references", expression = "java( mapReferences() )")
	OutSendMessageRequestV2 mapOutSendMessageRequestV2(Osi2Message message);

	@Mapping(target = "mimeType", source = "contentType")
	@Mapping(target = "isOriginalMessage", expression = "java( false )")
	@Named("mapMessageExchangeFile")
	MessageExchangeFiles mapMessageExchangeFile(AttachmentInfo fileUpload);

	default List<V1References> mapReferences() {
		return Collections.emptyList();
	}

	default V1ReplyBehavior mapReplyAction(PostfachNachricht.ReplyOption replyOption) {
		return switch (replyOption) {
			case FORBIDDEN -> V1ReplyBehavior.REPLYFORBIDDEN;
			case POSSIBLE -> V1ReplyBehavior.REPLYPOSSIBLE;
			case MANDATORY -> V1ReplyBehavior.REPLYMANDATORY;
		};
	}

	@Mapping(target = "target", constant = "UNSPECIFIED")
	@Mapping(target = "uploadUid", source = "upload.guid")
	@Mapping(target = "fileName", source = "upload.file.name")
	@Mapping(target = "contentType", source = "upload.file.contentType")
	@Mapping(target = "totalChunks", expression = "java( (int) fileChunkInfo.upload().numberOfChunks() )")
	@Mapping(target = "totalFileSize", source = "upload.file.size")
	DomainChunkMetaData mapDomainChunkMetaData(FileChunkInfo fileChunkInfo);
}
