package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import java.time.Duration;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.function.BooleanSupplier;

import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor(access = AccessLevel.PRIVATE)
public class WaitUtil {

	public static boolean waitUntil(BooleanSupplier condition, Duration interval, Duration timeout) {
		var waitResult = new AtomicBoolean(false);
		try (var executor = Executors.newSingleThreadScheduledExecutor()) {
			executor.scheduleAtFixedRate(() -> {
						if (condition.getAsBoolean()) {
							waitResult.set(true);
							executor.shutdown();
						}
					}, 0, interval.toMillis(), TimeUnit.MILLISECONDS)
					.get(timeout.toMillis(), TimeUnit.MILLISECONDS);
		} catch (InterruptedException e) {
			Thread.currentThread().interrupt();
			throw new IllegalStateException("[waitUntil] Interrupt!", e.getCause());
		} catch (TimeoutException | CancellationException e) {
			// ignore
		} catch (ExecutionException e) {
			if (e.getCause() instanceof RuntimeException runtimeException) {
				throw runtimeException;
			} else {
				throw new IllegalStateException("Unexpected exception while waiting for condition!", e.getCause());
			}
		}
		return waitResult.get();
	}
}
