package de.ozgcloud.nachrichten.postfach.osiv2;

import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public record HttpProxyConfig(
		String host,
		String port
) {

	public static Optional<HttpProxyConfig> fromEnv() {
		return Optional.ofNullable(System.getenv("HTTP_PROXY"))
				.map(HttpProxyConfig::fromEnvVar);
	}

	private static HttpProxyConfig fromEnvVar(String httpProxyEnvVar) {
		var matcher = matchProxyRegex(httpProxyEnvVar);
		return new HttpProxyConfig(
				matcher.group(1),
				matcher.group(2)
		);
	}

	private static Matcher matchProxyRegex(String text) {
		var matcher = Pattern.compile("([0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+):([0-9]+)").matcher(text);
		if (matcher.find()) {
			return matcher;
		}
		throw new IllegalArgumentException("Proxy host and port not found in '%s'".formatted(text));
	}
}
