package de.ozgcloud.nachrichten.postfach.osiv2;

import static de.ozgcloud.nachrichten.postfach.osiv2.config.Osi2PostfachProperties.*;

import org.mapstruct.factory.Mappers;
import org.springframework.boot.autoconfigure.AutoConfiguration;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;

import de.ozgcloud.apilib.common.callcontext.CallContext;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.apilib.file.OzgCloudFileService;
import de.ozgcloud.apilib.file.grpc.GrpcOzgCloudFileService;
import de.ozgcloud.apilib.file.grpc.OzgCloudFileMapper;
import de.ozgcloud.apilib.vorgang.OzgCloudUserIdMapper;
import de.ozgcloud.vorgang.grpc.binaryFile.BinaryFileServiceGrpc;
import net.devh.boot.grpc.client.inject.GrpcClient;

@SpringBootApplication
@AutoConfiguration
public class TestApplication {

	// From de.ozgcloud.nachrichten.common.grpc.NachrichtenCallContextAttachingInterceptor
	public static final String NACHRICHTEN_MANAGER_CLIENT_NAME = "OzgCloud_NachrichtenManager";
	public static final String NACHRICHTEN_MANAGER_SENDER_USER_ID = "system-nachrichten_manager-sender";

	@GrpcClient(GRPC_FILE_MANAGER_NAME)
	private BinaryFileServiceGrpc.BinaryFileServiceBlockingStub fileServiceBlockingStub;
	@GrpcClient(GRPC_FILE_MANAGER_NAME)
	private BinaryFileServiceGrpc.BinaryFileServiceStub fileServiceAsyncServiceStub;

	@Bean
	OzgCloudCallContextProvider ozgCloudCallContextProvider() {
		var userIdMapper = Mappers.getMapper(OzgCloudUserIdMapper.class);
		return () -> CallContext.builder().clientName(NACHRICHTEN_MANAGER_CLIENT_NAME)
				.userId(userIdMapper.toUserId(NACHRICHTEN_MANAGER_SENDER_USER_ID)).build();
	}

	@Bean(OZG_CLOUD_FILE_SERVICE_NAME)
	OzgCloudFileService grpcOzgCloudFileService(OzgCloudCallContextProvider contextProvider) {
		return new GrpcOzgCloudFileService(fileServiceBlockingStub, fileServiceAsyncServiceStub, contextProvider,
				Mappers.getMapper(OzgCloudFileMapper.class));
	}
}
