package de.ozgcloud.nachrichten.postfach.osiv2.attachment;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;

import java.io.ByteArrayInputStream;
import java.util.stream.Stream;

import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import lombok.SneakyThrows;

class ConcatInputStreamTest {

	@DisplayName("should concatenate input streams")
	@Test
	@SneakyThrows
	void shouldConcatenateInputStreams() {
		var concatStream = createExampleConcatInputStream();

		var result = IOUtils.toByteArray(concatStream);

		assertEquals("123456789", new String(result));
	}

	@DisplayName("should read one")
	@Test
	@SneakyThrows
	void shouldReadOne() {
		var concatStream = createExampleConcatInputStream();

		var result = concatStream.read();

		assertThat(result).isEqualTo('1');
	}

	@DisplayName("should write to buffer with offset")
	@Test
	@SneakyThrows
	void shouldWriteToBufferWithOffset() {
		var buffer = new byte[10];
		var concatStream = createExampleConcatInputStream();

		var result = concatStream.read(buffer, 2, 2);
		assertThat(result).isEqualTo(2);
		assertThat(buffer).containsExactly(0, 0, '1', '2', 0, 0, 0, 0, 0, 0);

		result = concatStream.read(buffer, 3, 2);
		assertThat(result).isEqualTo(2);
		assertThat(buffer).containsExactly(0, 0, '1', '3', '4', 0, 0, 0, 0, 0);

		result = concatStream.read(buffer, 4, 2);
		assertThat(result).isEqualTo(2);
		assertThat(buffer).containsExactly(0, 0, '1', '3', '5', '6', 0, 0, 0, 0);

		result = concatStream.read(buffer, 5, 2);
		assertThat(result).isEqualTo(2);
		assertThat(buffer).containsExactly(0, 0, '1', '3', '5', '7', '8', 0, 0, 0);

		result = concatStream.read(buffer, 6, 2);
		assertThat(result).isEqualTo(1);
		assertThat(buffer).containsExactly(0, 0, '1', '3', '5', '7', '9', 0, 0, 0);

		result = concatStream.read(buffer, 7, 2);
		assertThat(result).isEqualTo(-1);
		assertThat(buffer).containsExactly(0, 0, '1', '3', '5', '7', '9', 0, 0, 0);
	}

	private ConcatInputStream createExampleConcatInputStream() {
		var streams = Stream.of("1234", "567", "", "89")
				.map(s -> new ByteArrayInputStream(s.getBytes()))
				.toList();
		return new ConcatInputStream(streams.iterator());
	}

}