package de.ozgcloud.nachrichten.postfach.osiv2.attachment;

import static org.junit.jupiter.api.Assertions.*;

import java.io.ByteArrayInputStream;

import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import lombok.SneakyThrows;

class LimitedInputStreamTest {

	@DisplayName("should limit input stream")
	@Test
	@SneakyThrows
	void shouldLimitInputStream() {
		var inputStream = createExampleLimitedInputStream();

		var result = IOUtils.toByteArray(inputStream);

		assertEquals("12345", new String(result));
	}

	@DisplayName("should read one")
	@Test
	@SneakyThrows
	void shouldReadOne() {
		var inputStream = createExampleLimitedInputStream();

		var result = inputStream.read();

		assertEquals('1', result);
	}

	@DisplayName("should keep parent stream usable")
	@Test
	@SneakyThrows
	void shouldKeepParentStreamUsable() {
		var parentStream = new ByteArrayInputStream("123456789".getBytes());
		var inputStream = new LimitedInputStream(parentStream, 5);
		var inputStream2 = new LimitedInputStream(parentStream, 5);
		IOUtils.toByteArray(inputStream);

		var result = IOUtils.toByteArray(inputStream2);

		assertEquals("6789", new String(result));
	}

	private LimitedInputStream createExampleLimitedInputStream() {
		return new LimitedInputStream(new ByteArrayInputStream("123456789".getBytes()), 5);
	}
}