package de.ozgcloud.nachrichten.postfach.osiv2.attachment;

import static de.ozgcloud.nachrichten.postfach.osiv2.factory.Osi2FileUploadTestFactory.*;
import static de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.apilib.file.OzgCloudFile;
import de.ozgcloud.apilib.file.OzgCloudUploadFile;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.AttachmentFileTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.GrpcOzgFileTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.model.AttachmentFile;
import de.ozgcloud.vorgang.grpc.file.GrpcOzgFile;

class Osi2AttachmentFileMapperTest {

	private final Osi2AttachmentFileMapper mapper = Mappers.getMapper(Osi2AttachmentFileMapper.class);

	@DisplayName("map to upload file metadata")
	@Nested
	class TestMapToUploadFileMetadata {

		private final GrpcOzgFile grpcOzgFile = GrpcOzgFileTestFactory.create();

		@DisplayName("should map id")
		@Test
		void shouldMapId() {
			var result = doMapping();

			assertThat(result.getId()).hasToString(grpcOzgFile.getId());
		}

		@DisplayName("should map name")
		@Test
		void shouldMapName() {
			var result = doMapping();

			assertThat(result.getName()).isEqualTo(grpcOzgFile.getName());
		}

		@DisplayName("should map contentType")
		@Test
		void shouldMapContentType() {
			var result = doMapping();

			assertThat(result.getContentType()).isEqualTo(grpcOzgFile.getContentType());
		}

		@DisplayName("should map size")
		@Test
		void shouldMapSize() {
			var result = doMapping();

			assertThat(result.getSize()).isEqualTo(grpcOzgFile.getSize());
		}

		private OzgCloudFile doMapping() {
			return mapper.mapToUploadFileMetadata(grpcOzgFile);
		}
	}

	@DisplayName("to ozg cloud upload file")
	@Nested
	class TestToOzgCloudUploadFile {
		private final AttachmentFile attachmentFile = AttachmentFileTestFactory.create();

		@DisplayName("should map fileName")
		@Test
		void shouldMapFileName() {
			var result = doMapping();

			assertThat(result.getFileName()).isEqualTo(UPLOAD_NAME);
		}

		@DisplayName("should map contentType")
		@Test
		void shouldMapContentType() {
			var result = doMapping();

			assertThat(result.getContentType()).isEqualTo(UPLOAD_CONTENT_TYPE);
		}

		@DisplayName("should map fieldName")
		@Test
		void shouldMapFieldName() {
			var result = doMapping();

			assertThat(result.getFieldName()).isEqualTo(Osi2AttachmentFileMapper.ATTACHMENT_FIELD_NAME);
		}

		@DisplayName("should map vorgangId")
		@Test
		void shouldMapVorgangId() {
			var result = doMapping();

			assertThat(result.getVorgangId()).isEqualTo(VORGANG_ID);
		}

		private OzgCloudUploadFile doMapping() {
			return mapper.toOzgCloudUploadFile(attachmentFile);
		}
	}

}