package de.ozgcloud.nachrichten.postfach.osiv2.extension;

import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory.*;

import java.util.Optional;

import jakarta.annotation.Nullable;

import org.junit.jupiter.api.extension.AfterAllCallback;
import org.junit.jupiter.api.extension.AfterEachCallback;
import org.junit.jupiter.api.extension.BeforeAllCallback;
import org.junit.jupiter.api.extension.BeforeEachCallback;
import org.junit.jupiter.api.extension.ExtensionContext;

import com.github.tomakehurst.wiremock.WireMockServer;

import de.ozgcloud.nachrichten.postfach.osiv2.factory.JwtFactory;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Getter
@RequiredArgsConstructor
public class OsiMockServerExtension implements BeforeAllCallback, AfterAllCallback, BeforeEachCallback {

	private WireMockServer serviceKontoMockServer;
	private WireMockServer postfachFacadeMockServer;

	@Override
	public void beforeAll(ExtensionContext context) {
		setupPostfachFacadeMock();
		setupServiceKontoMock();
	}

	@Override
	public void afterAll(ExtensionContext context) {
		if (serviceKontoMockServer != null) {
			serviceKontoMockServer.shutdown();
			serviceKontoMockServer = null;
		}

		if (postfachFacadeMockServer != null) {
			postfachFacadeMockServer.shutdown();
			postfachFacadeMockServer = null;
		}
	}

	private void setupPostfachFacadeMock() {
		postfachFacadeMockServer = setupWiremockServer(postfachFacadeMockServer, 32813);
	}

	private void setupServiceKontoMock() {
		serviceKontoMockServer = setupWiremockServer(serviceKontoMockServer, 32812);
	}

	private WireMockServer setupWiremockServer(@Nullable WireMockServer existingServer, int port) {
		if (existingServer != null && existingServer.isRunning()) {
			existingServer.resetAll();
			return existingServer;
		} else {
			var server = new WireMockServer(port);
			server.start();
			return server;
		}
	}

	public String getAccessTokenUrl() {
		return serviceKontoMockServer.baseUrl() + "/access-token";
	}

	public String getPostfachFacadeUrl() {
		return Optional.ofNullable(postfachFacadeMockServer)
				.map(WireMockServer::baseUrl)
				.orElseThrow(() -> new IllegalStateException("PostfachFacadeMockServer not initialized"));
	}

	@Override
	public void beforeEach(ExtensionContext context) {
		setupPostfachFacadeMock();
		setupServiceKontoMock();

		serviceKontoMockServer.stubFor(post("/access-token")
				.withHeader("Content-Type", equalTo("application/x-www-form-urlencoded"))
				.withFormParam("grant_type", equalTo("client_credentials"))
				.withFormParam("client_id", equalTo(CLIENT_ID))
				.withFormParam("scope", equalTo(String.join(" ", CLIENT_SCOPES)))
				.withFormParam("resource", equalTo(RESOURCE_URN))
				.willReturn(okJson(JwtFactory.createTokenResponse(
						JwtFactory.createAccessTokenExampleWithExpireIn(900)
				))));
	}
}
