package de.ozgcloud.nachrichten.postfach.osiv2.factory;

import static de.ozgcloud.nachrichten.postfach.osiv2.factory.JsonUtil.*;

import java.util.List;
import java.util.Map;


import de.ozgcloud.nachrichten.postfach.osiv2.extension.Jwt;
import lombok.SneakyThrows;

public class JwtFactory {

	public static final String CLIENT_ID = "ITCase OZG Kopfstelle";
	public static final String RESOURCE_URN = "urn:dataport:osi:postfach:rz2:itcase:sh";
	public static final List<String> CLIENT_SCOPES = List.of("default", "access_urn:itcase:scope:ozgkopfstelle");

	public static Jwt createAccessTokenExampleWithExpireIn(Integer expireInSeconds) {
		var nowEpochSeconds = System.currentTimeMillis() / 1000;
		return Jwt.create(
				Map.of(
						"alg", "RS256",
						"kid", "...",
						"x5t", "...",
						"typ", "at+jwt"
				),
				Map.of(
						"iss", "https://idp.serviceportal-stage.schleswig-holstein.de/webidp2",
						"nbf", nowEpochSeconds,
						"iat", nowEpochSeconds,
						"exp", nowEpochSeconds + expireInSeconds,
						"aud", RESOURCE_URN,
						"scope", String.join(" ", CLIENT_SCOPES),
						"client_id", CLIENT_ID,
						"client_tenant", "urn:osp:names:realm:stage:sh",
						"jti", "..."
				),
				"signature".getBytes());
	}

	@SneakyThrows
	public static String createTokenResponse(Jwt accessToken) {
		return toJson(Map.of(
				"access_token", accessToken.token(),
				"token_type", "Bearer",
				"expires_in", accessToken.expireTime() - accessToken.issuedAt(),
				"scope", accessToken.scope()
		));
	}

}
