package de.ozgcloud.nachrichten.postfach.osiv2.factory;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;

import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.DTOMailboxScopeData;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MddiMailboxIndexData;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MddiMailboxType;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.ResponsesMailboxDirectoryResponse;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1Meta;

public class ResponsesMailboxDirectoryResponseTestFactory {

	public static final String NAME_ID = UUID.randomUUID().toString();
	public static final String MAILBOX_GUID_SH = UUID.randomUUID().toString();
	public static final String MAILBOX_GUID_HH = UUID.randomUUID().toString();
	public static final String TENANT_SH = "SH";
	public static final String TENANT_HH = "HH";

	public static ResponsesMailboxDirectoryResponse create() {
		return createBuilder(
				buildMailboxWithSHTenant().build(),
				buildMailboxWithHHTenant().build()
		).build();
	}

	public static ResponsesMailboxDirectoryResponse.Builder createBuilder(MddiMailboxIndexData... mailBoxes) {
		return ResponsesMailboxDirectoryResponse.builder()
				.mailBoxIndex(List.of(createScopeData(mailBoxes)))
				.meta(V1Meta.builder().build());
	}

	public static MddiMailboxIndexData.Builder buildMailboxWithSHTenant() {
		return MddiMailboxIndexData.builder()
				.mailboxguid(UUID.fromString(MAILBOX_GUID_SH))
				.tenant(TENANT_SH)
				.mailboxname("")
				.mailboxdescription("")
				.mailboxtype(MddiMailboxType.PERSONAL);
	}

	public static MddiMailboxIndexData.Builder buildMailboxWithHHTenant() {
		return MddiMailboxIndexData.builder()
				.mailboxguid(UUID.fromString(MAILBOX_GUID_HH))
				.tenant(TENANT_HH)
				.mailboxtype(MddiMailboxType.PERSONAL);
	}

	private static DTOMailboxScopeData createScopeData(MddiMailboxIndexData... mailBoxes) {
		return DTOMailboxScopeData.builder()
				.membershipcontext(UUID.fromString(NAME_ID))
				.memberscope(Arrays.asList(mailBoxes))
				.build();
	}
}
