package de.ozgcloud.nachrichten.postfach.osiv2.factory;

import java.time.OffsetDateTime;
import java.time.ZonedDateTime;
import java.util.List;
import java.util.UUID;

import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1EidasLevel;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyBehavior;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyMessage;

public class V1ReplyMessageTestFactory {

	public static final String SEQUENCE_NUMMER = "OZG-Cloud-VorgangId";
	public static final String SUBJECT = "Das ist das Subject";
	public static final String HTML_REPLY_BODY = """
			Das ist das Multiline&amp;&lt;b&gt;a&lt;/b&gt<br><br/>
			Body""";
	public static final String REPLY_BODY = """
			Das ist das Multiline&&lt;b&gt;a&lt;/b&gt<br><br/>
			Body""";
	public static final String MESSAGE_ID = UUID.randomUUID().toString();
	public static final String MESSAGE_BOX_ID = UUID.randomUUID().toString();
	public static final ZonedDateTime RESPONSE_TIME = ZonedDateTime.now();

	public static V1ReplyMessage create() {
		return createBuilder().build();
	}

	public static V1ReplyMessage.Builder createBuilder() {
		return V1ReplyMessage.builder()
				.sequencenumber(SEQUENCE_NUMMER)
				.subject(SUBJECT)
				.replyAction(V1ReplyBehavior.REPLYPOSSIBLE)
				.isObligatory(false)
				.eidasLevel(V1EidasLevel.LOW)
				.isHtml(false)
				.body(REPLY_BODY)
				.guid(UUID.fromString(MESSAGE_ID))
				.files(List.of(V1ReplyFilesTestFactory.create()))
				.messageBox(UUID.fromString(MESSAGE_BOX_ID))
				.responseTime(OffsetDateTime.of(RESPONSE_TIME.toLocalDateTime(), RESPONSE_TIME.getOffset()));
	}
}
