package de.ozgcloud.nachrichten.postfach.osiv2.model;

import static de.ozgcloud.nachrichten.postfach.osiv2.model.Osi2Attachment.*;
import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;

import de.ozgcloud.apilib.file.OzgCloudFileTestFactory;

public class Osi2AttachmentTest {

	@DisplayName("calculate number of chunks")
	@Nested
	class TestCalculateNumberOfChunks {

		@DisplayName("should return zero")
		@Test
		void shouldReturnZero() {
			var upload = createWithSize(0L);

			var result = upload.numberOfChunks();

			assertThat(result).isZero();
		}

		@DisplayName("should return one")
		@ParameterizedTest
		@ValueSource(longs = { 1, CHUNK_SIZE - 1, CHUNK_SIZE })
		void shouldReturnOne(long fileLength) {
			var upload = createWithSize(fileLength);

			var result = upload.numberOfChunks();

			assertThat(result).isEqualTo(1L);
		}

		@DisplayName("should return two")
		@ParameterizedTest
		@ValueSource(longs = { CHUNK_SIZE + 1, CHUNK_SIZE * 2 - 1, CHUNK_SIZE * 2 })
		void shouldReturnTwo(long fileLength) {
			var upload = createWithSize(fileLength);

			var result = upload.numberOfChunks();

			assertThat(result).isEqualTo(2L);
		}

		private Osi2Attachment createWithSize(long size) {
			return Osi2Attachment.builder()
					.file(OzgCloudFileTestFactory.createBuilder()
							.size(size)
							.build())
					.build();
		}
	}
}
